#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735
#include <SPI.h>

#define R      16
#define G      4
#define B      2

#define TFT_CS        27
#define TFT_RST       26
#define TFT_A0        25
#define TFT_SDA       33
#define TFT_SCK       32

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_A0, TFT_SDA, TFT_SCK, TFT_RST);

const uint16_t  Black        = 0x0000;
const uint16_t  Blue         = 0x001F;
const uint16_t  Red          = 0xF800;
const uint16_t  Green        = 0x07E0;
const uint16_t  Cyan         = 0x07FF;
const uint16_t  Magenta      = 0xF81F;
const uint16_t  Yellow       = 0xFFE0;
const uint16_t  White        = 0xFFFF;

String utf8rus(String source)
{
  int i, k;
  String target;
  unsigned char n;
  char m[2] = {};
  k = source.length(); i = 0;

  while (i < k) {
    n = source[i]; i++;
    if (n >= 0xBF) {
      switch (n) {
        case 0xD0: {
            n = source[i]; i++;
            if (n == 0x81) {
              n = 0xA8;
              break;
            }
            if (n >= 0x90 && n <= 0xBF) n = n + 0x2F;
            break;
          }
        case 0xD1: {
            n = source[i]; i++;
            if (n == 0x91) {
              n = 0xB7;
              break;
            }
            if (n >= 0x80 && n <= 0x8F) n = n + 0x6F;
            break;
          }
      }
    }
    m[0] = n; target = target + String(m);
  }
  return target;
}



String utf8rus2(String source)
{
  int i, k;
  String target;
  unsigned char n;
  char m[2] = { '0', '\0' };

  k = source.length(); i = 0;

  while (i < k) {
    n = source[i]; i++;

    if (n >= 0xC0) {
      switch (n) {
        case 0xD0: {
            n = source[i]; i++;
            if (n == 0x81) {
              n = 0xA8;
              break;
            }
            if (n >= 0x90 && n <= 0xBF) n = n + 0x30;
            break;
          }
        case 0xD1: {
            n = source[i]; i++;
            if (n == 0x91) {
              n = 0xB8;
              break;
            }
            if (n >= 0x80 && n <= 0x8F) n = n + 0x70;
            break;
          }
      }
    }
    m[0] = n; target = target + String(m);
  }
  return target;
}

#define BUZZER_PIN 19
#define BUTTON_PIN 21
#include <EncButton.h>
EncButton<EB_TICK, BUTTON_PIN> btn;

int strt;

void setup() {
  Serial.begin(115200); //Bluetooth device name
  delay(1000);
  pinMode(14, OUTPUT);
  pinMode(18, OUTPUT);
  pinMode(BUZZER_PIN, OUTPUT);
  pinMode(BUTTON_PIN, INPUT);
  pinMode(R, OUTPUT);
  pinMode(G, OUTPUT);
  pinMode(B, OUTPUT);
  digitalWrite(14, HIGH);

  tft.initR(INITR_BLACKTAB);


  tft.fillScreen(Black);
  tft.setTextWrap(false);
  tft.setCursor(0, 1);
  tft.setTextColor(White);
  tft.setTextSize(5);
  tft.println(utf8rus("под"));
  tft.println(utf8rus("ключ"));
  tft.println(utf8rus("ение"));
  while (not Serial.available()) {
    btn.tick();
  }
  while (Serial.available()) {
    Serial.read();
  }
  tft.fillScreen(Black);
  tft.setTextWrap(false);
  tft.setCursor(0, 1);
  tft.println(utf8rus("жми"));
  strt = millis();
  while (not btn.click() && millis() - strt <= 10000) {
    btn.tick();
  }

  Serial.print(1);
  tft.fillScreen(Black);
  tft.setTextWrap(false);
  tft.setCursor(0, 1);
  tft.setTextSize(4);
  tft.println(utf8rus("наст-"));
  tft.println(utf8rus("ройка"));
  while (not Serial.available()) {
    btn.tick();
  }
  while (Serial.available()) {
    Serial.read();
  }
}
int st = millis();
bool flag = true;

int right_ans = 0;
int wrong_ans = 0;
String letter;

void loop() {
  if (Serial.available()) {
    int val = (Serial.read());
    if (val == 50) {
      digitalWrite(BUZZER_PIN, LOW);
      digitalWrite(R, LOW);
      digitalWrite(G, LOW);
      digitalWrite(B, HIGH);
    }
    else if (val == 51) {
      digitalWrite(R, HIGH);
      digitalWrite(G, LOW);
      digitalWrite(B, LOW);
      wrong_ans++;
    }
    else if (val == 48) {
      digitalWrite(R, LOW);
      digitalWrite(G, HIGH);
      digitalWrite(B, LOW);
      digitalWrite(BUZZER_PIN, HIGH);
      right_ans++;
    }
    else if (val == 49) {
      if (millis() - st >= 500) {
        st = millis();
        flag = !flag;
        if (flag) {
          digitalWrite(R, LOW);
          digitalWrite(G, LOW);
          digitalWrite(B, HIGH);
        } else {
          digitalWrite(R, LOW);
          digitalWrite(G, LOW);
          digitalWrite(B, LOW);
        }
      }
    }
    else if (val == 52) {
      letter = utf8rus("с");
      tft.setTextWrap(false);
      tft.fillScreen(Black);
      tft.setCursor(40, 1);
      tft.setTextColor(White);
      tft.setTextSize(9);
      tft.print(letter);
      tft.setTextColor(Green);
      tft.setCursor(5, 80);
      tft.print(right_ans);
      tft.setTextColor(Red);
      tft.setCursor(65, 80);
      tft.print(wrong_ans);
      Serial.println(right_ans, wrong_ans);
    }
    else if (val == 53) {
      letter = utf8rus("т");
      tft.setTextWrap(false);
      tft.fillScreen(Black);
      tft.setCursor(40, 1);
      tft.setTextColor(White);
      tft.setTextSize(9);
      tft.print(letter);
      tft.setTextColor(Green);
      tft.setCursor(5, 80);
      tft.print(right_ans);
      tft.setTextColor(Red);
      tft.setCursor(65, 80);
      tft.print(wrong_ans);
    }
    else {
      letter = utf8rus(String(char(val + 96)));
      tft.setTextWrap(false);
      tft.fillScreen(Black);
      tft.setCursor(40, 1);
      tft.setTextColor(White);
      tft.setTextSize(9);
      tft.print(letter);
      tft.setTextColor(Green);
      tft.setCursor(5, 80);
      tft.print(right_ans);
      tft.setTextColor(Red);
      tft.setCursor(65, 80);
      tft.print(wrong_ans);
    }
  }
  if (right_ans + wrong_ans >= 9) {
    digitalWrite(BUZZER_PIN, LOW);
    tft.setTextWrap(false);
    tft.fillScreen(Black);
    tft.setCursor(0, 0);
    tft.setTextColor(White);
    tft.setTextSize(6);
    tft.print(utf8rus("итог"));
    tft.setTextSize(9);
    tft.setTextColor(Green);
    tft.setCursor(5, 80);
    tft.print(right_ans);
    tft.setTextColor(Red);
    tft.setCursor(65, 80);
    tft.print(wrong_ans);
    //    while (not btn.click()) {
    //      btn.tick();
    //    }
    delay(7000);
    right_ans = 0;
    wrong_ans = 0;
    Serial.println(1);
  }
}